/*******************************************************************************
 * linked_list
 *
 * A simple doubly linked list class in C.
 *
 *******************************************************************************
 *
 * Copyright (C) 2010 Sarah Eris Horsley Caffee
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *******************************************************************************
 *
 * Not thread safe.
 *
 * Since this class does not attempt to manage memory for the actual stored 
 * objects, there is no "clear" method to delete the entire list.  Instead,
 * the user must pop the entire list and delete the individually returned
 * objects one at a time.
 *
 * A potential improvement would be to allow the registration of an object
 * destructor at list creation which could then be used by a clear() method.
 *
 *
 *
 *
 * OOH SHINY!
 * http://tinobox.com/wordpress/object-oriented-c-coding/
 *
 * Looks liks good stuff.  Must read.
 *
 ******************************************************************************/

#ifndef LINKEDLIST_H
#define LINKEDLIST_H

typedef struct ll_node ll_node;

typedef struct LinkedList LinkedList;
struct LinkedList
   {
   /* Public */
   void *       (* push)  (LinkedList * this, void * data);
   void *       (* pop)   (LinkedList * this);
   void *       (* insert)(LinkedList * this, void * data);
   void *	(* remove)(LinkedList * this);
   void *       (* insert_at_head)(LinkedList * this, void * data);
   int		(* size)  (LinkedList * this);
   void	*	(* head)  (LinkedList * this);
   void	*	(* tail)  (LinkedList * this);
   void	*	(* next)  (LinkedList * this);
   void	*	(* prev)  (LinkedList * this);
   void *	(* data)  (LinkedList * this);

   /* Private (notionally, at least) */
   ll_node *	Head;
   ll_node *	Curr;
   ll_node *	Tail;
   int		Size;
   };

/* Forward declarations for more traditional C usage. */

void   ll_init  (LinkedList * this);
void * ll_push  (LinkedList * this, void * data);
void * ll_pop   (LinkedList * this);
void * ll_insert(LinkedList * this, void * data);
void * ll_remove(LinkedList * this);
void * ll_insert_at_head(LinkedList * this, void * data);
int    ll_size  (LinkedList * this);
void * ll_head  (LinkedList * this);
void * ll_tail  (LinkedList * this);
void * ll_next  (LinkedList * this);
void * ll_prev  (LinkedList * this);
void * ll_data  (LinkedList * this);

#endif

